' iMacros Self-Test Script
' Copyright © 1992-2021 Progress Software Corporation and/or one of its subsidiaries or affiliates.
' All rights reserved.

Option Explicit

Const ERR_USERSTOPPED	= -101
Const ERR_BROWSERCLOSED	= -102

' Init the iMacros scripting interface
Dim iim1, ret
Set iim1 = CreateObject("iMacros")

' Init the test configuration
Dim TestConfig
Set TestConfig = New CTestConfig

Call MsgBox("iMacros Self Test Script" & vbNewLine & vbNewLine &_
	"Scripting Interface Version: " & iim1.iimGetInterfaceVersion(), _
	vbOkOnly, WScript.ScriptName)

' Browser switches can be passed to the script via the command line
Dim browserSwitches, browser

If WScript.Arguments.Count > 0 Then
	Dim arg, args
	For Each arg In WScript.Arguments
		args = args & arg & " "
	Next
	browserSwitches = Trim(args)
End If

' Determine if a browser was specified on the command line
Dim regEx, matches
Set regEx = New RegExp

regEx.Pattern = Join(TestConfig.BrowserCodes, "|")
regEx.Global = False
regEx.IgnoreCase = True

Set matches = regEx.Execute(browserSwitches)

If matches.Count > 0 Then
	browser = LCase(Replace(matches(0).Value, "-", ""))
End If

If Len(browser) = 0 Then
	If WScript.Arguments.Count = 0 Then
		' No browser was specified on the command line so prompt the user to choose a browser
		browser = TestConfig.PromptUserForBrowser()

		If browser <> "v12" Then
			browserSwitches = "-" & browser
		End If
	Else
		browser = "v12"
	End If
End If

TestConfig.Browser = browser

' Get the collection of test macros
Dim TestMacros
Set TestMacros = TestConfig.Macros

If vbCancel = MsgBox("Click OK to start the test run of " & TestMacros.Count & " macros in " & TestConfig.GetBrowserName(TestConfig.Browser), vbOkCancel+vbExclamation, WScript.ScriptName) Then
	WScript.Quit()
End If

' Open the browser
ret = iim1.iimOpen(CStr(browserSwitches))
If ret < 0 Then
	Call MsgBox(iim1.iimGetErrorText(), vbOkOnly+vbCritical, WScript.ScriptName)
	WScript.Quit(ret)
End If

' Create the output file
Dim fso, outputFile
Set fso = TestConfig.FileSystemObject
Set outputFile = fso.CreateTextFile(TestConfig.OutputFileName, True)

outputFile.WriteLine("*** Test Report for iMacros Self Test Script ***")
outputFile.WriteLine("*** Scripting Interface Version: " & iim1.iimGetInterfaceVersion() & " ***" & vbNewLine)

' Run the tests
Dim i : i = 1
Dim macroName
For Each macroName In TestMacros.Keys()
	iim1.iimDisplay(macroName)
	ret = iim1.iimPlay(TestMacros(macroName), 60)
	outputFile.WriteLine(i & " ; " & Time() & " ; " & macroName & " ; " & iim1. iimGetErrorText())

	' Exit the loop if the user clicked stop or closed the browser
	If ret = ERR_USERSTOPPED Or ret = ERR_BROWSERCLOSED Then
		Exit For
	End If

	i = i + 1
Next

outputFile.Close()

Call MsgBox("Test complete!" & vbNewLine & vbNewLine &_
	"Report Name: " & TestConfig.OutputFileName & vbNewLine & vbNewLine &_
	"Click OK to close the browser and exit.", _
	vbOkOnly+vbExclamation, WScript.ScriptName)

iim1.iimClose()
WScript.Quit(ret)

'=====================================================================
' TestConfig Class
'=====================================================================
Class CTestConfig

	' -----------------
	' Private members
	' -----------------
	Private m_Browsers         ' Collection of browser names keyed by code
	Private m_browser          ' The browser code of the browser chosen for the test
	Private m_excludedMacros   ' Array of macros to exclude based on browser
	Private m_fso              ' FileSystemObject
	Private m_outputFileName   ' The name of the file to write test results
	Private m_TestMacros       ' Collection of macros to test

	Private Sub Class_Initialize()
		
		Set m_Browsers = CreateObject("Scripting.Dictionary")

		m_Browsers("ng")  = "iMacros browser"
		m_Browsers("v12") = "iMacros classic browser (v7-v12)"
		m_Browsers("cr")  = "Chrome"
		m_Browsers("fx")  = "Firefox"
		m_Browsers("ie")  = "IE"

		m_excludedMacros = Array(_
			Array("Draw",                        Array("ng", "cr", "fx")), _
			Array("Detect Javascript Errors",    Array("ng", "cr", "fx")), _
			Array("DirectScreen",                Array("cr", "fx")), _
			Array("DirectScreen Click",          Array("ng", "v12", "cr", "fx", "ie")), _
			Array("DirectScreen Relative Click", Array("cr", "fx")), _
			Array("DragDrop",                    Array("cr", "fx")), _
			Array("Eval",                        Array("ng", "v12", "cr", "fx", "ie")), _
			Array("FillForm Using Events",       Array("fx")), _
			Array("ImageDownload",               Array("ng")), _
			Array("ImageRecognition",            Array("cr", "fx")), _
			Array("JavascriptDialog",            Array("cr", "fx")), _
			Array("Flash",                       Array("ng", "cr", "fx")), _
			Array("Loop-Csv-2-Web",              Array("ng", "v12", "cr", "fx", "ie")), _
			Array("Pop-ups",                     Array("ng", "v12", "cr", "fx", "ie")), _
			Array("SavePdf",                     Array("ng", "cr", "fx")), _
			Array("Upload",                      Array("fx")), _
			Array("WebPageDialog",               Array("ng", "cr", "fx")))
			
		Set m_fso = Nothing
		Set m_TestMacros = Nothing

	End Sub

	Private Sub Class_Terminate()
		
		Set m_Browsers = Nothing
		Set m_fso = Nothing
		Set m_TestMacros = Nothing
		
	End Sub
	
	' -----------------
	' Constants
	' -----------------
	Public Property Get ERR_CRITICAL()
		ERR_CRITICAL = -1
	End Property

	' -----------------
	' Public properties
	' -----------------
	Public Property Get Browser()
		Browser = m_browser
	End Property

	Public Property Let Browser(value)
		m_browser = value
	End Property

	Public Property Get BrowserCodes()
		BrowserCodes = m_Browsers.Keys()
	End Property

	Public Property Get FileSystemObject()

		If m_fso Is Nothing Then
			Set m_fso = CreateObject("Scripting.FileSystemObject")
		End If

		Set FileSystemObject = m_fso

	End Property

	Public Property Set FileSystemObject(fso)
		Set m_fso = fso
	End Property

	Public Property Get OutputFileName()

		If Len(m_outputFileName) = 0 Then
			If Len(m_browser) = 0 Then
				Call Err.Raise(ERR_CRITICAL, "OutputFileName", "TestConfig.Browser was not specified")
			End If

			Select Case m_browser
				Case "ng"
					m_outputFileName = "self-test-im.txt"
				Case "v12"
					m_outputFileName = "self-test-im-classic.txt"
				Case Else
					m_outputFileName = "self-test-" & m_browser & ".txt"
			End Select
		End If

		OutputFileName = m_outputFileName

	End Property

	Public Property Get Macros()
		
		If m_TestMacros Is Nothing Then
			If Len(m_browser) = 0 Then
				Call Err.Raise(ERR_CRITICAL, "Macros", "TestConfig.Browser was not specified")
			End If

			If m_fso Is Nothing Then
				Set m_fso = CreateObject("Scripting.FileSystemObject")
			End If

			' Get the current folder
			Dim mypath : mypath = Left(WScript.ScriptFullName, InStrRev(WScript.ScriptFullName, "\"))

			' Get the path to the demo macros
			Dim demoFolderPath : demoFolderPath = m_fso.GetParentFolderName(m_fso.GetParentFolderName(mypath)) & "\Macros\Demo\"

			If Not m_fso.FolderExists(demoFolderPath) Then
				Call Err.Raise(ERR_CRITICAL, "m_TestMacros", demoFolderPath & " does not exist")
			End If

			' Generate the collection of macros (key = macro name, value = macro path)
			Set m_TestMacros = CreateObject("Scripting.Dictionary")

			Dim demoFolder
			Set demoFolder = m_fso.GetFolder(demoFolderPath)

			Dim demoFile
			For Each demoFile In demoFolder.Files
				If LCase(m_fso.GetExtensionName(demoFile.Name)) = "iim" Then
					m_TestMacros(demoFile.Name) = demoFile.Path
				End If
			Next

			' Remove any excluded macros for the specified browser
			Dim excludedMacro
			For Each excludedMacro In m_excludedMacros
				If InStr(Join(excludedMacro(1), "|"), m_browser) > 0 Then
					On Error Resume Next
					m_TestMacros.Remove(excludedMacro(0) & ".iim")
					On Error Goto 0
				End If
			Next
		End If

		Set Macros = m_TestMacros

	End Property

	' -----------------
	' Public methods
	' -----------------
	Public Function GetBrowserName(browserCode)
		If Not m_Browsers.Exists(browserCode) Then
			Call Err.Raise(ERR_CRITICAL, "BrowserName", "Invalid browser code: " & browserCode)
		End If
		GetBrowserName = m_Browsers(browserCode)
	End Function

	Public Function PromptUserForBrowser()

		Dim ValidChoices
		Set ValidChoices = CreateObject("Scripting.Dictionary")

		Dim prompt : prompt = "Please enter the number of the browser you want to test:" & vbNewLine & vbNewLine
		Dim i : i = 1

		Dim browserCode
		For Each browserCode In BrowserCodes
			prompt = prompt & i & " => " & GetBrowserName(browserCode) & vbNewLine
			ValidChoices(CStr(i)) = browserCode
			i = i + 1
		Next

		Dim response : response = Trim(InputBox(prompt, WScript.ScriptName, "1"))

		If Len(response) = 0 Then
			' User clicked Cancel so quit
			WScript.Quit()
		End If

		If Not ValidChoices.Exists(response) Then
			Call MsgBox("Invalid browser selection: " & response & vbNewLine & vbNewLine &_
				"Please enter 1 - " & ValidChoices.Count, vbOkOnly+vbCritical, WScript.ScriptName)
			WScript.Quit()
		End If

		PromptUserForBrowser = ValidChoices(response)

	End Function

End Class
