' iMacros File-2-Web-Method2 Script
' Copyright © 1992-2021 Progress Software Corporation and/or one of its subsidiaries or affiliates.
' All rights reserved.

Option Explicit

Const ERR_USERSTOPPED	= -101
Const ERR_BROWSERCLOSED	= -102

' Init the iMacros scripting interface
Dim iim1, ret
Set iim1 = CreateObject("iMacros")

Call MsgBox("This script demonstrates how to read data from a CSV file and submit this information to a website. " &_
	"Unlike the file-2-web.vbs example, this script reads the data directly from the file (iim-test-submit.csv) and " &_
	"passes it to the Wsh-Submit-2-Web macro." & vbNewLine & vbNewLine &_
	"Tip: This script has the same function as database-2-web.vbs but uses a text file instead of a database as input.", _
	vbOkOnly, WScript.ScriptName)

' Get the current folder
Dim mypath : mypath = Left(WScript.ScriptFullName, InStrRev(WScript.ScriptFullName, "\"))

' Browser switches can be passed to the script via the command line
Dim browserSwitches : browserSwitches = "-ng"
Dim browserIsOpen : browserIsOpen = False

If WScript.Arguments.Count > 0 Then
	Dim arg, args
	For Each arg In WScript.Arguments
		args = args & arg & " "
	Next
	browserSwitches = Trim(args)
End If

' Open the browser
ret = iim1.iimOpen(CStr(browserSwitches))
If ret < 0 Then DisplayErrorAndQuit(iim1.iimGetErrorText())
browserIsOpen = True

' Init the input file
Dim InputFile
Set InputFile = New CCSVFileReader

InputFile.FileName = "iim-test-submit.csv"
InputFile.Folder = mypath
InputFile.ColumnNames = InputFile.HEADER_ROW ' derive the column/field names from the header row
InputFile.StartRow = 2 ' skip the header row when reading the file

' Loop through the file one line/row at a time
Do While InputFile.Read()
	iim1.iimDisplay("Submitting data from row " & InputFile.Row)

	' Pass the values from the input file to the macro
	Dim colName
	For Each colName In InputFile.ColumnNames
		Call iim1.iimSet(Replace(colName, " ", "-"), InputFile(colName))
	Next

	' Play the macro
	ret = iim1.iimPlay(mypath & "Macros\Wsh-Submit-2-Web.iim")

	If ret = ERR_USERSTOPPED Or ret = ERR_BROWSERCLOSED Then
		' The user clicked the stop button or closed the browser, so abort the script
		If ret = ERR_BROWSERCLOSED Then browserIsOpen = False
		DisplayErrorAndQuit("Playback was interrupted by the user." & vbNewLine & vbNewLine & "Click OK to exit the script.")
	ElseIf ret < 0 Then
	   ' Some other unexpected error occurred, so abort the script. Rather than abort,
	   ' you could instead e.g. log the error to a file (see combine-macros.vbs) and
	   ' attempt to keep processing the input file.
	   DisplayErrorAndQuit(iim1.iimGetErrorText())
	End If
Loop

iim1.iimDisplay("Done!")
Call MsgBox("All records were submitted successfully!" & vbNewLine & vbNewLine &_
	"Press OK to close the browser and exit.", _
	vbOkOnly+vbInformation, WScript.ScriptName)

CleanUp()
WScript.Quit(ret)

Sub DisplayError(errMsg)
	Call MsgBox(errMsg, vbOkOnly+vbExclamation, WScript.ScriptName)
End Sub

Sub DisplayErrorAndQuit(errMsg)
	DisplayError(errMsg)
	CleanUp()
	WScript.Quit(ret)
End Sub

Sub CleanUp()
	If browserIsOpen Then
		If iim1.iimClose() < 0 Then DisplayError(iim1.iimGetErrorText())
	End If
End Sub

'=====================================================================
' CSVFileReader Class
'	A lightweight, self-contained class for sequentially reading a
'	CSV file.
'=====================================================================
Class CCSVFileReader

	' -----------------
	' Private members
	' -----------------
	Private m_Columns			' Collection of column names and values representing the most recent line/row read from the file
	Private m_delimiter			' The separtor character that delimits columns in the file
	Private m_endRow			' Specifies the last line/row to read in the file
	Private m_file				' The file object
	Private m_fileFormat		' The encoding format (ASCII/Unicode) of the file
	Private m_fileName			' The name of the file
	Private m_folder			' The path to the file
	Private m_row				' The number of the most recent row read from the file
	Private m_startRow			' Specifies the first line/row to start reading from the file
	Private m_useHeaderRow		' Specifies whether to derive the column names from the first row in the file
	
	Private Sub Class_Initialize()
		
		Set m_Columns = CreateObject("Scripting.Dictionary")
		m_Columns.CompareMode = vbTextCompare

		Set m_file = Nothing

		m_delimiter = ","
		m_endRow = -1
		m_fileFormat = FILEFORMAT_ASCII
		m_row = 0
		m_startRow = 1
		
	End Sub
	
	Private Sub Class_Terminate()
		
		If Not m_file Is Nothing Then
			m_file.Close()
			Set m_file = Nothing
		End If
		
		Set m_Columns = Nothing
		
	End Sub
	
	Private Function ConcatPath(ByVal leftSide, ByVal rightSide)
		' Concatenates two portions of a file path
		Dim sep : sep = "\"
		
		If Len(leftSide) = 0 Or Len(rightSide) = 0 Then
			sep = ""
		End If
		
		Dim newLeft : newLeft = leftSide
		Dim newRight : newRight = rightSide
		
		If Right(newLeft, 1) = "\" Then
			newLeft = Left(newLeft, Len(newLeft) - 1)
		End If
		
		If Left(newRight, 1) = "\" Then
			newRight = Right(newRight, Len(newRight) - 1)
		End If
		
		ConcatPath = newLeft & sep & newRight
	End Function

	Private Function CountChar(str, char)
		' Returns the number of occurrences of char in str

		Dim pos : pos = 0
		CountChar = -1

		Do 
			CountChar = CountChar + 1
			pos = InStr(pos + 1, str, char)
		Loop Until pos = 0

	End Function

	Private Sub OpenFile()

		If IsOpen Then
			Exit Sub
		End If

		If Len(m_fileName) = 0 Then
			Call Err.Raise(ERR_CRITICAL, "OpenFile", "FileName is not specified")
		End If
	
		If Len(m_folder) = 0 Then
			Call Err.Raise(ERR_CRITICAL, "OpenFile", "Folder is not specified")
		End If
	
		If m_Columns.Count = 0 And Not m_useHeaderRow Then
			Call Err.Raise(ERR_CRITICAL, "OpenFile", "Column names are not specified")
		End If
	
		Dim fso
		Set fso = CreateObject("Scripting.FileSystemObject")
		Set m_file = Nothing
		
		On Error Resume Next
		Set m_file = fso.OpenTextFile(FullPath, OPENFILE_FORREADING, m_fileFormat)
		
		Dim errNumber : errNumber = Err.Number
		On Error Goto 0
		
		If m_file Is Nothing Then
			Call Err.Raise(ERR_CRITICAL, "OpenFile", FullPath & " was not found or could not be opened (" & errNumber & ")")
		End If
		
		If m_file.AtEndOfStream Then
			Call Err.Raise(ERR_CRITICAL, "OpenFile", "File is empty")
		End If
		
		m_row = 0
		
		If m_useHeaderRow Then
			' Read the column names from the header row
			Dim cols : cols = SplitDSVLine(ReadRow(), m_delimiter)
			Dim colName
			
			cols(0) = StripBOM(cols(0))

			For Each colName In cols
				m_Columns(Trim(colName)) = Null
			Next
			
			' Close and reopen the file
			m_file.Close()
			Set m_file = fso.OpenTextFile(FullPath, OPENFILE_FORREADING, m_fileFormat)
		End If

		If m_startRow > 1 Then
			' Advance the file to the specified start row
			Do While m_row < m_startRow - 1 And Not m_file.AtEndOfStream
				ReadRow()
				m_row = m_row + 1
			Loop

			If m_file.AtEndOfStream Then
				Call Err.Raise(ERR_CRITICAL, "OpenFile", "StartRow is greater than the number of rows in the file")
			End If
		End If
		
		Set fso = Nothing
	
	End Sub
	
	Private Function ReadRow()
		' Reads a logical row from the file. A row may span more than one line if any of the
		' columns contain embedded newline characters. The assumption this method makes is
		' that any column values containing a newline are well formed by being enclosed in
		' double-quotes.
		
		Dim done : done = False
		Dim quotes : quotes = 0
		Dim line, row

		Do While Not done And Not m_file.AtEndOfStream
			line = m_file.ReadLine()

			If Len(row) > 0 Then
				row = row & vbNewLine
			End If

			row = row & line
			quotes = quotes + CountChar(line, """")

			If quotes Mod 2 = 0 Then
				done = True
			End If
		Loop

		ReadRow = row

	End Function

	Private Function SplitDSVLine(line, sep)
		' Split a line into columns delimited by the specified sep character, where each 
		' column might have enclosing quotation marks (also with embedded sep chars).
		' Returns an array of the split values.

		ReDim columns(0)
		Dim i, j, state, c, lineLen

		' state:
		' 0 = not quotation mark; set by final sep char
		' 1 = quotation mark started
		' 2 = quotation mark end but not yet final sep char
		' 3 = no quotation mark expected (first character was no quotation mark)

		j = 0
		state = 0
		columns(0) = ""
		lineLen = Len(line)
		
		For i = 1 To lineLen
			
			c = Mid(line, i, 1)
			
			Select Case state
				Case 0
					Select Case c
						Case sep
							j = j + 1
							Redim Preserve columns(j)
							columns(j) = ""		' empty column
						Case """"
							state = 1
						Case " "				' ignore leading spaces
						Case Else
							columns(j) = columns(j) & c
							state = 3
					End Select
				Case 1
					Select Case c
						Case """"
							state = 2
						Case Else
							columns(j) = columns(j) & c
					End Select
				Case 2
					Select Case c
						Case """"				' embedded double-quote
							columns(j) = columns(j) & c
							state = 1
						Case sep
							j = j + 1
							Redim Preserve columns(j)
							columns(j) = ""
							state = 0
						Case " "				' ignore trailing spaces
						Case Else
							' error
					End Select
				Case 3
					Select Case c
						Case sep
							j = j + 1
							Redim Preserve columns(j)
							columns(j) = ""
							state = 0
						Case Else
							columns(j) = columns(j) & c
					End Select
			End Select
		Next
		
		SplitDSVLine = columns
		
	End Function

	Private Function StripBOM(str)
		StripBOM = Replace(str, Chr(&HEF) & Chr(&HBB) & Chr(&HBF), "")
	End Function
	
	' -----------------
	' Constants
	' -----------------
	Public Property Get ERR_CRITICAL()
		ERR_CRITICAL = -1
	End Property
	
	Public Property Get FILEFORMAT_ASCII()
		FILEFORMAT_ASCII = 0
	End Property
	
	Public Property Get FILEFORMAT_SYSTEMDEFAULT()
		FILEFORMAT_SYSTEMDEFAULT = -2
	End Property
	
	Public Property Get FILEFORMAT_UNICODE()
		FILEFORMAT_UNICODE = -1
	End Property
	
	Public Property Get HEADER_ROW()
		HEADER_ROW = 1
	End Property
	
	Public Property Get OPENFILE_FORREADING()
		OPENFILE_FORREADING = 1
	End Property
	
	' -----------------
	' Public properties
	' -----------------
	Public Default Property Get Item(columnName)
		Item = m_Columns(columnName)
	End Property
	
	Public Property Let Item(columnName, value)
		m_Columns(columnName) = value
	End Property
	
	Public Property Get ColumnNames()
		ColumnNames = m_Columns.Keys()
	End Property
	
	Public Property Let ColumnNames(namesArray)

		Dim invalidType : invalidType = "namesArray must be an Array of column names, or HEADER_ROW to use the column names specified in the file"
		
		m_Columns.RemoveAll()
		
		If IsArray(namesArray) Then
			Dim i
			For i = 0 To UBound(namesArray)
				m_Columns(namesArray(i)) = Null
			Next
		ElseIf VarType(namesArray) = vbInteger Then
			If namesArray = HEADER_ROW Then
				m_useHeaderRow = True
			Else
				Call Err.Raise(ERR_CRITICAL, "ColumnNames", invalidType)
			End If
		Else
			Call Err.Raise(ERR_CRITICAL, "ColumnNames", invalidType)
		End If
		
	End Property
	
	Public Property Get Delimiter()
		Delimiter = m_delimiter
	End Property
	
	Public Property Let Delimiter(value)
		m_delimiter = value
	End Property

	Public Property Get EndRow()
		EndRow = m_endRow
	End Property
	
	Public Property Let EndRow(value)
		m_endRow = value
	End Property
	
	Public Property Get FileFormat()
		FileFormat = m_fileFormat
	End Property
	
	Public Property Let FileFormat(value)
		m_fileFormat = value
	End Property
	
	Public Property Get FileName()
		FileName = m_fileName
	End Property
	
	Public Property Let FileName(value)
		m_fileName = value
	End Property
	
	Public Property Get Folder()
		Folder = m_folder
	End Property
	
	Public Property Let Folder(value)
		m_folder = value
	End Property
	
	Public Property Get FullPath()
		FullPath = ConcatPath(m_folder, m_fileName)
	End Property

	Public Property Get IsOpen()
		IsOpen = (Not m_file Is Nothing)
	End Property
	
	Public Property Get Row()
		Row = m_row
	End Property
	
	Public Property Get StartRow()
		StartRow = m_startRow
	End Property
	
	Public Property Let StartRow(value)
		m_startRow = value
	End Property
	
	' -----------------
	' Public methods
	' -----------------
	Public Function Read()
	
		If Not IsOpen Then
			OpenFile()
		End If
		
		If m_endRow > 0 And m_row >= m_endRow Then
			Read = False
			Exit Function
		End If
		
		If m_file.AtEndOfStream Then
			Read = False
			Exit Function
		End If
		
		Dim cols : cols = SplitDSVLine(ReadRow(), m_delimiter)
		m_row = m_row + 1

		If m_row = 1 Then
			cols(0) = StripBOM(cols(0))
		End If
		
		Dim colNames : colNames = m_Columns.Keys()
		Dim i

		For i = 0 To UBound(colNames)
			If i > UBound(cols) Then
				Exit For
			End If
		
			m_Columns(colNames(i)) = cols(i)
		Next
		
		Read = True
	
	End Function
	
	Public Sub Rewind()
		' Rewinds the file to m_startRow
		If IsOpen Then
			m_file.Close()
			Set m_file = Nothing
			OpenFile()
		End If
	End Sub
	
End Class
